@@ -18,28 +18,28 @@ Anansi::DatabaseComponent - A manager template for database drivers.
             INPUT => [
                 'some text',
                 {
-                    NAME => 'someParameter'
+                    NAME => 'someParameter',
                 }, {
                     INPUT => [
                         'more text',
                         {
-                            NAME => 'anotherParameter'
+                            NAME => 'anotherParameter',
                         },
-                        'yet more text'
+                        'yet more text',
                     ]
                 }, {
                     DEFAULT => 'abc',
-                    NAME => 'yetAnotherParameter'
-                }
+                    NAME => 'yetAnotherParameter',
+                },
             ],
-            someParameter => 12345,
-            anotherParameter => 'blah blah blah'
+            (%parameters),
         );
     }
 
     sub validate {
         my ($self, $channel, %parameters) = @_;
-        return Anansi::DatabaseComponent::validate(undef, DRIVER => 'Example');
+        $parameters{DRIVER} = 'Example';
+        return Anansi::DatabaseComponent::validate(undef, %parameters);
     }
 
     Anansi::Component::addChannel('Anansi::Database::Example', 'AUTOCOMMIT' => 'Anansi::DatabaseComponent::autocommit');
@@ -55,129 +55,184 @@ Anansi::DatabaseComponent - A manager template for database drivers.
 
     1;
 
+    package main;
+
+    use Anansi::Database;
+
+    my $database = Anansi::Database->new();
+    my $component = $database->addComponent(undef,
+        DRIVER => 'Example',
+    );
+    if(defined($component)) {
+        if($database->connect(
+            undef,
+            $component,
+            someParameter => 'some data',
+            anotherParameter => 'more data',
+            yetAnotherParameter => 'further data',
+        )) {
+            my $result = $database->statement(
+                undef,
+                $component,
+                SQL => 'SELECT someThing FROM someTable where modified = ?;',
+                INPUT => [
+                    {
+                        NAME => 'modified',
+                    },
+                ],
+                modified => '2011-02-22 00:21:46',
+            );
+            if(!defined($result)) {
+            } elsif(ref($result) =~ /^ARRAY$/i) {
+                foreach my $record (@{$result}) {
+                    next if(ref($record) !~ /^HASH$/i);
+                    print 'someThing: "'.${$record}{someThing}.'"'."\n";
+                }
+            }
+        }
+    }
+
+    1;
+
 =head1 DESCRIPTION
 
 Manages a database connection providing generic processes to allow it's opening,
-closing and various SQL interactions.  Uses L<Anansi::Actor>,
-L<Anansi::Component> and L<base>.
+closing and various SQL interactions.  Uses L<Anansi::Actor>.
 
 =cut
 
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use base qw(Anansi::Component);
 
 use Anansi::Actor;
 
 
-=head1 INHERITED METHODS
-
-=cut
-
-
-=head2 addChannel
-
-Declared in L<Anansi::Component>.
-
-=cut
-
-
-=head2 channel
-
-Declared in L<Anansi::Component>.
-
-=cut
-
-
-=head2 componentManagers
-
-Declared in L<Anansi::Component>.
-
-=cut
-
-
-=head2 finalise
-
-    $OBJECT->SUPER::finalise();
-
-Declared in L<Anansi::Class>.  Overridden by this module.
-
-=cut
-
-
-sub finalise {
-    my ($self, %parameters) = @_;
-    $self->finish();
-    $self->disconnect();
-}
-
-
-=head2 implicate
-
-Declared in L<Anansi::Class>.  Intended to be overridden by an extending module.
-
-=cut
-
-
-=head2 import
-
-Declared in L<Anansi::Class>.
-
-=cut
-
-
-=head2 initialise
-
-    $OBJECT->SUPER::initialise();
-
-Declared in L<Anansi::Class>.  Overridden by this module.
-
-=cut
-
-
-sub initialise {
-    my ($self, %parameters) = @_;
-    Anansi::Actor->new(
-        PACKAGE => 'DBI',
-    );
-    $self->{STATEMENT} = {};
-}
-
-
-=head2 old
-
-Declared in L<Anansi::Class>.
-
-=cut
-
-
-=head2 removeChannel
-
-Declared in L<Anansi::Component>.
-
-=cut
-
-
-=head2 used
-
-Declared in L<Anansi::Class>.
-
-=cut
-
-
-=head2 uses
-
-Declared in L<Anansi::Class>.
-
-=cut
-
-
 =head1 METHODS
 
 =cut
 
 
+=head2 Anansi::Class
+
+See L<Anansi::Class|Anansi::Class> for details.  A parent module of L<Anansi::Component|Anansi::Component>.
+
+=cut
+
+
+=head3 DESTROY
+
+See L<Anansi::Class::DESTROY|Anansi::Class/"DESTROY"> for details.
+
+=cut
+
+
+=head3 finalise
+
+See L<Anansi::Class::finalise|Anansi::Class/"finalise"> for details.  Overridden by L<Anansi::DatabaseComponent::finalise|Anansi::DatabaseComponent/"finalise">.  A virtual method.
+
+=cut
+
+
+=head3 implicate
+
+See L<Anansi::Class::implicate|Anansi::Class/"implicate"> for details.  A virtual method.
+
+=cut
+
+
+=head3 import
+
+See L<Anansi::Class::import|Anansi::Class/"import"> for details.
+
+=cut
+
+
+=head3 initialise
+
+See L<Anansi::Class::initialise|Anansi::Class/"initialise"> for details.  A virtual method.
+
+=cut
+
+
+=head3 new
+
+See L<Anansi::Class::new|Anansi::Class/"new"> for details.
+
+=cut
+
+
+=head3 old
+
+See L<Anansi::Class::old|Anansi::Class/"old"> for details.
+
+=cut
+
+
+=head3 used
+
+See L<Anansi::Class::used|Anansi::Class/"used"> for details.
+
+=cut
+
+
+=head3 uses
+
+See L<Anansi::Class::uses|Anansi::Class/"uses"> for details.
+
+=cut
+
+
+=head3 using
+
+See L<Anansi::Class::using|Anansi::Class/"using"> for details.
+
+=cut
+
+
+=head2 Anansi::Component
+
+See L<Anansi::Component|Anansi::Component> for details.  A parent module of L<Anansi::DatabaseComponent|Anansi::DatabaseComponent>.
+
+=cut
+
+
+=head3 Anansi::Class
+
+See L<Anansi::Class|Anansi::Class> for details.  A parent module of L<Anansi::Component|Anansi::Component>.
+
+=cut
+
+
+=head3 addChannel
+
+See L<Anansi::Component::addChannel|Anansi::Component/"addChannel"> for details.
+
+=cut
+
+
+=head3 channel
+
+See L<Anansi::Component::channel|Anansi::Component/"channel"> for details.
+
+=cut
+
+
+=head3 componentManagers
+
+See L<Anansi::Component::componentManagers|Anansi::Component/"componentManagers"> for details.
+
+=cut
+
+
+=head3 removeChannel
+
+See L<Anansi::Component::removeChannel|Anansi::Component/"removeChannel"> for details.
+
+=cut
+
+
 =head2 autoCommit
 
     if(1 == Anansi::DatabaseComponent::autocommit($OBJECT, undef));
@@ -508,10 +563,16 @@ Named parameters.
 
 =over 4
 
-=item INPUT I<(Array B<or> Scalar, Required)>
+=item HANDLE I<(DBI::db, Optional)>
+
+The database handle of an existing database connection.  Used in preference to
+the I<INPUT> parameter.
+
+=item INPUT I<(Array B<or> Scalar, Optional)>
 
 An array or single value containing a description of each parameter in the order
-that it is passed to the database driver's I<connect> method.
+that it is passed to the database driver's I<connect> method.  Used when the
+I<HANDLE> parameter does not exist.
 
 =over 4
 
@@ -556,8 +617,9 @@ The data types used to validate the value to use.
 
 =back
 
-Attempts to perform a database connection using the supplied parameters.
-Returns B<1> I<(one)> on success and B<0> I<(zero)> on failure.
+Either uses an existing database connection or attempts to perform a database
+connection using the supplied parameters.  Returns B<1> I<(one)> on success and
+B<0> I<(zero)> on failure.
 
 =cut
 
@@ -566,76 +628,119 @@ sub connect {
     my ($self, $channel, %parameters) = @_;
     return 0 if(ref($self) =~ /^(|ARRAY|CODE|FORMAT|GLOB|HASH|IO|LVALUE|REF|Regexp|SCALAR|VSTRING)$/i);
     $self->disconnect();
-    return 0 if(!defined($parameters{INPUT}));
-    return 0 if(ref($parameters{INPUT}) !~ /^ARRAY$/i);
-    my @inputs;
-    foreach my $input (@{$parameters{INPUT}}) {
-        if(ref($input) !~ /^HASH$/i) {
-            push(@inputs, $input);
-            next;
-        }
-        my $value = undef;
-        $value = ${$input}{DEFAULT} if(defined(${$input}{DEFAULT}));
-        if(!defined(${$input}{NAME})) {
-            if(!defined(${$input}{INPUT})) {
-            } elsif(ref(${$input}{INPUT}) !~ /^ARRAY$/i) {
-            } elsif(!defined(${$input}{REF})) {
-            } elsif(ref(${$input}{REF}) !~ /^$/i) {
-            } elsif('' eq ${$input}{REF}) {
-                my @subInputs;
-                for(my $index = 0; $index < scalar(@{${$input}{INPUT}}); $index++) {
-                    if(ref(${${$input}{INPUT}}[$index]) =~ /^$/i) {
-                        push(@subInputs, ${${$input}{INPUT}}[$index]);
-                        next;
-                    } elsif(ref(${${$input}{INPUT}}[$index]) !~ /^HASH$/) {
-                        next;
-                    }
-                    my $subValue = '';
-                    $subValue = ${${${$input}{INPUT}}[$index]}{DEFAULT} if(defined(${${${$input}{INPUT}}[$index]}{DEFAULT}));
-                    if(!defined(${${${$input}{INPUT}}[$index]}{NAME})) {
-                    } elsif(ref(${${${$input}{INPUT}}[$index]}{NAME}) !~ /^$/) {
-                    } elsif(defined($parameters{${${${$input}{INPUT}}[$index]}{NAME}})) {
-                        if(!defined(${${${$input}{INPUT}}[$index]}{REF})) {
-                            $subValue = $parameters{${${${$input}{INPUT}}[$index]}{NAME}} if('' eq ref($parameters{${${${$input}{INPUT}}[$index]}{NAME}}));
-                        } elsif(ref(${${${$input}{INPUT}}[$index]}{REF}) !~ /^$/) {
-                        } elsif('' ne ${${${$input}{INPUT}}[$index]}{REF}) {
-                        } elsif('' ne ref($parameters{${${${$input}{INPUT}}[$index]}{NAME}})) {
-                        } else {
-                            $subValue = $parameters{${${${$input}{INPUT}}[$index]}{NAME}};
-                        }
-                    }
-                    push(@subInputs, $subValue);
-                }
-                $value = join('', @subInputs);
-            } elsif(${$input}{REF} =~ /^HASH$/i) {
-                my %subInputs;
-                foreach my $subInput (@{${$input}{INPUT}}) {
-                    next if(ref($subInput) !~ /^HASH$/i);
-                    my $subValue = undef;
-                    $subValue = ${$subInput}{DEFAULT} if(defined(${$subInput}{DEFAULT}));
-                    if(!defined(${$subInput}{NAME})) {
-                    } elsif(ref(${$subInput}{NAME}) !~ /^$/) {
-                    } elsif(defined($parameters{${$subInput}{NAME}})) {
-                        if(!defined(${$subInput}{REF})) {
-                        } elsif(ref(${$subInput}{REF}) =~ /^ARRAY$/i) {
-                            my %refs = map { $_ => 1 } (@{${$subInput}{REF}});
-                            $subValue = $parameters{${$subInput}{NAME}} if(defined($refs{ref($parameters{${$subInput}{NAME}})}));
-                        } elsif(ref(${$subInput}{REF}) !~ /^$/) {
-                        } elsif(${$subInput}{REF} ne ref($parameters{${$subInput}{NAME}})) {
-                        } else {
-                            $subValue = $parameters{${$subInput}{NAME}};
-                        }
-                    }
-                    $subInputs{${$subInput}{NAME}} = $subValue;
-                }
-                $value = \%subInputs;
+    if(defined($parameters{HANDLE})) {
+        return 0 if(ref($parameters{HANDLE}) !~ /^DBI::db$/);
+        $self->{HANDLE} = $parameters{HANDLE};
+        $self->{MANAGE_HANDLE} = 0;
+    } elsif(!defined($parameters{INPUT})) {
+        return 0;
+    } elsif(ref($parameters{INPUT}) !~ /^ARRAY$/i) {
+        return 0;
+    } else {
+        my @inputs;
+        foreach my $input (@{$parameters{INPUT}}) {
+            if(ref($input) !~ /^HASH$/i) {
+                push(@inputs, $input);
+                next;
             }
-        } elsif(ref(${$input}{NAME}) !~ /^$/) {
-        } elsif(defined($parameters{${$input}{NAME}})) {
-            if(!defined(${$input}{REF})) {
-            } elsif(ref(${$input}{REF}) =~ /^ARRAY$/i) {
-                my %refs = map { $_ => 1 } (@{${$input}{REF}});
-                if(!defined($refs{ref($parameters{${$input}{NAME}})})) {
+            my $value = undef;
+            $value = ${$input}{DEFAULT} if(defined(${$input}{DEFAULT}));
+            if(!defined(${$input}{NAME})) {
+                if(!defined(${$input}{INPUT})) {
+                } elsif(ref(${$input}{INPUT}) !~ /^ARRAY$/i) {
+                } elsif(!defined(${$input}{REF})) {
+                } elsif(ref(${$input}{REF}) !~ /^$/i) {
+                } elsif('' eq ${$input}{REF}) {
+                    my @subInputs;
+                    for(my $index = 0; $index < scalar(@{${$input}{INPUT}}); $index++) {
+                        if(ref(${${$input}{INPUT}}[$index]) =~ /^$/i) {
+                            push(@subInputs, ${${$input}{INPUT}}[$index]);
+                            next;
+                        } elsif(ref(${${$input}{INPUT}}[$index]) !~ /^HASH$/) {
+                            next;
+                        }
+                        my $subValue = '';
+                        $subValue = ${${${$input}{INPUT}}[$index]}{DEFAULT} if(defined(${${${$input}{INPUT}}[$index]}{DEFAULT}));
+                        if(!defined(${${${$input}{INPUT}}[$index]}{NAME})) {
+                        } elsif(ref(${${${$input}{INPUT}}[$index]}{NAME}) !~ /^$/) {
+                        } elsif(defined($parameters{${${${$input}{INPUT}}[$index]}{NAME}})) {
+                            if(!defined(${${${$input}{INPUT}}[$index]}{REF})) {
+                                $subValue = $parameters{${${${$input}{INPUT}}[$index]}{NAME}} if('' eq ref($parameters{${${${$input}{INPUT}}[$index]}{NAME}}));
+                            } elsif(ref(${${${$input}{INPUT}}[$index]}{REF}) !~ /^$/) {
+                            } elsif('' ne ${${${$input}{INPUT}}[$index]}{REF}) {
+                            } elsif('' ne ref($parameters{${${${$input}{INPUT}}[$index]}{NAME}})) {
+                            } else {
+                                $subValue = $parameters{${${${$input}{INPUT}}[$index]}{NAME}};
+                            }
+                        }
+                        push(@subInputs, $subValue);
+                    }
+                    $value = join('', @subInputs);
+                } elsif(${$input}{REF} =~ /^HASH$/i) {
+                    my %subInputs;
+                    foreach my $subInput (@{${$input}{INPUT}}) {
+                        next if(ref($subInput) !~ /^HASH$/i);
+                        my $subValue = undef;
+                        $subValue = ${$subInput}{DEFAULT} if(defined(${$subInput}{DEFAULT}));
+                        if(!defined(${$subInput}{NAME})) {
+                        } elsif(ref(${$subInput}{NAME}) !~ /^$/) {
+                        } elsif(defined($parameters{${$subInput}{NAME}})) {
+                            if(!defined(${$subInput}{REF})) {
+                            } elsif(ref(${$subInput}{REF}) =~ /^ARRAY$/i) {
+                                my %refs = map { $_ => 1 } (@{${$subInput}{REF}});
+                                $subValue = $parameters{${$subInput}{NAME}} if(defined($refs{ref($parameters{${$subInput}{NAME}})}));
+                            } elsif(ref(${$subInput}{REF}) !~ /^$/) {
+                            } elsif(${$subInput}{REF} ne ref($parameters{${$subInput}{NAME}})) {
+                            } else {
+                                $subValue = $parameters{${$subInput}{NAME}};
+                            }
+                        }
+                        $subInputs{${$subInput}{NAME}} = $subValue;
+                    }
+                    $value = \%subInputs;
+                }
+            } elsif(ref(${$input}{NAME}) !~ /^$/) {
+            } elsif(defined($parameters{${$input}{NAME}})) {
+                if(!defined(${$input}{REF})) {
+                } elsif(ref(${$input}{REF}) =~ /^ARRAY$/i) {
+                    my %refs = map { $_ => 1 } (@{${$input}{REF}});
+                    if(!defined($refs{ref($parameters{${$input}{NAME}})})) {
+                    } elsif(ref($parameters{${$input}{NAME}}) !~ /^HASH$/i) {
+                        $value = $parameters{${$input}{NAME}};
+                    } else {
+                        if(!defined(${$input}{INPUT})) {
+                            $value = $parameters{${$input}{NAME}};
+                        } elsif(ref(${$input}{INPUT}) !~ /^HASH$/i) {
+                            $value = $parameters{${$input}{NAME}};
+                        } else {
+                            my %subInputs;
+                            foreach my $subInput (keys(%{${$input}{INPUT}})) {
+                                if(ref($subInput) !~ /^HASH$/i) {
+                                    $subInputs{$subInput} = $subInput;
+                                    next;
+                                }
+                                my $subValue = undef;
+                                $value = ${${${$input}{INPUT}}{$subInput}}{DEFAULT} if(defined(${${${$input}{INPUT}}{$subInput}}{DEFAULT}));
+                                if(!defined(${${${$input}{INPUT}}{$subInput}}{NAME})) {
+                                } elsif(ref(${${${$input}{INPUT}}{$subInput}}{NAME}) !~ /^$/) {
+                                } elsif(defined($parameters{${${${$input}{INPUT}}{$subInput}}{NAME}})) {
+                                    if(!defined(${${${$input}{INPUT}}{$subInput}}{REF})) {
+                                    } elsif(ref(${${${$input}{INPUT}}{$subInput}}{REF}) =~ /^ARRAY$/i) {
+                                        my %refs = map { $_ => 1 } (@{${${${$input}{INPUT}}{$subInput}}{REF}});
+                                        $subValue = $parameters{${${${$input}{INPUT}}{$subInput}}{NAME}} if(defined($refs{ref($parameters{${${${$input}{INPUT}}{$subInput}}{NAME}})}));
+                                    } elsif(ref(${${${$input}{INPUT}}{$subInput}}{REF}) !~ /^$/) {
+                                    } elsif(${${${$input}{INPUT}}{$subInput}}{REF} ne ref($parameters{${${${$input}{INPUT}}{$subInput}}{NAME}})) {
+                                    } else {
+                                        $subValue = $parameters{${${${$input}{INPUT}}{$subInput}}{NAME}};
+                                    }
+                                }
+                                $subInputs{$subInput} = $subValue;
+                            }
+                            $value = \%subInputs;
+                        }
+                    }
+                } elsif(ref(${$input}{REF}) !~ /^$/) {
+                } elsif(${$input}{REF} ne ref($parameters{${$input}{NAME}})) {
                 } elsif(ref($parameters{${$input}{NAME}}) !~ /^HASH$/i) {
                     $value = $parameters{${$input}{NAME}};
                 } else {
@@ -645,9 +750,9 @@ sub connect {
                         $value = $parameters{${$input}{NAME}};
                     } else {
                         my %subInputs;
-                        foreach my $subInput (keys(%{${$input}{INPUT}})) {
+                        foreach my $key (keys(%{${$input}{INPUT}})) {
                             if(ref($subInput) !~ /^HASH$/i) {
-                                $subInputs{$subInput} = $subInput;
+                                push(@subInputs, $subInput);
                                 next;
                             }
                             my $subValue = undef;
@@ -670,49 +775,15 @@ sub connect {
                         $value = \%subInputs;
                     }
                 }
-            } elsif(ref(${$input}{REF}) !~ /^$/) {
-            } elsif(${$input}{REF} ne ref($parameters{${$input}{NAME}})) {
-            } elsif(ref($parameters{${$input}{NAME}}) !~ /^HASH$/i) {
-                $value = $parameters{${$input}{NAME}};
-            } else {
-                if(!defined(${$input}{INPUT})) {
-                    $value = $parameters{${$input}{NAME}};
-                } elsif(ref(${$input}{INPUT}) !~ /^HASH$/i) {
-                    $value = $parameters{${$input}{NAME}};
-                } else {
-                    my %subInputs;
-                    foreach my $key (keys(%{${$input}{INPUT}})) {
-                        if(ref($subInput) !~ /^HASH$/i) {
-                            push(@subInputs, $subInput);
-                            next;
-                        }
-                        my $subValue = undef;
-                        $value = ${${${$input}{INPUT}}{$subInput}}{DEFAULT} if(defined(${${${$input}{INPUT}}{$subInput}}{DEFAULT}));
-                        if(!defined(${${${$input}{INPUT}}{$subInput}}{NAME})) {
-                        } elsif(ref(${${${$input}{INPUT}}{$subInput}}{NAME}) !~ /^$/) {
-                        } elsif(defined($parameters{${${${$input}{INPUT}}{$subInput}}{NAME}})) {
-                            if(!defined(${${${$input}{INPUT}}{$subInput}}{REF})) {
-                            } elsif(ref(${${${$input}{INPUT}}{$subInput}}{REF}) =~ /^ARRAY$/i) {
-                                my %refs = map { $_ => 1 } (@{${${${$input}{INPUT}}{$subInput}}{REF}});
-                                $subValue = $parameters{${${${$input}{INPUT}}{$subInput}}{NAME}} if(defined($refs{ref($parameters{${${${$input}{INPUT}}{$subInput}}{NAME}})}));
-                            } elsif(ref(${${${$input}{INPUT}}{$subInput}}{REF}) !~ /^$/) {
-                            } elsif(${${${$input}{INPUT}}{$subInput}}{REF} ne ref($parameters{${${${$input}{INPUT}}{$subInput}}{NAME}})) {
-                            } else {
-                                $subValue = $parameters{${${${$input}{INPUT}}{$subInput}}{NAME}};
-                            }
-                        }
-                        $subInputs{$subInput} = $subValue;
-                    }
-                    $value = \%subInputs;
-                }
             }
+            push(@inputs, $value);
         }
-        push(@inputs, $value);
+        return 0 if(0 == scalar(@inputs));
+        my $handle = DBI->connect(@inputs);
+        return 0 if(!defined($handle));
+        $self->{HANDLE} = $handle;
+        $self->{MANAGE_HANDLE} = 1;
     }
-    return 0 if(0 == scalar(@inputs));
-    my $handle = DBI->connect(@inputs);
-    return 0 if(!defined($handle));
-    $self->{HANDLE} = $handle;
     return 1;
 }
 
@@ -755,14 +826,35 @@ sub disconnect {
     my ($self, $channel, %parameters) = @_;
     return 0 if(ref($self) =~ /^(|ARRAY|CODE|FORMAT|GLOB|HASH|IO|LVALUE|REF|Regexp|SCALAR|VSTRING)$/i);
     return 0 if(!defined($self->{HANDLE}));
-    $self->{HANDLE}->disconnect();
-    delete $self->{HANDLE};
+    if(!defined($self->{MANAGE_HANDLE})) {
+        $self->{MANAGE_HANDLE} = 0;
+    } elsif(1 == $self->{MANAGE_HANDLE}) {
+        $self->{HANDLE}->disconnect();
+        $self->{MANAGE_HANDLE} = 0;
+        delete $self->{HANDLE};
+    } else {
+        delete $self->{HANDLE};
+    }
     return 1;
 }
 
 Anansi::Component::addChannel('Anansi::DatabaseComponent', 'DISCONNECT' => 'disconnect');
 
 
+=head2 finalise
+
+Overrides L<Anansi::Class::finalise|Anansi::Class/"finalise">.  A virtual method.
+
+=cut
+
+
+sub finalise {
+    my ($self, %parameters) = @_;
+    $self->finish();
+    $self->disconnect();
+}
+
+
 =head2 finish
 
     if(1 == Anansi::DatabaseComponent::finish($OBJECT, undef));
@@ -849,7 +941,9 @@ Anansi::Component::addChannel('Anansi::DatabaseComponent', 'FINISH' => 'finish')
 
     my $HANDLE = $OBJECT->handle(undef);
 
-    my $HANDLE = $OBJECT->channel('HANDLE');
+    my $dbh = DBI->connect('DBI:mysql:database=someDatabase', 'someUser', 'somePassword');
+    my $HANDLE = $OBJECT->channel('HANDLE', $dbh);
+    if(defined($HANDLE));
 
 =over 4
 
@@ -881,7 +975,9 @@ sub handle {
             $self->finish();
             $self->disconnect();
         }
+        return if(ref($handle) !~ /^DBI::db$/);
         $self->{HANDLE} = $handle;
+        $self->{MANAGE_HANDLE} = 0;
     }
     return $self->{HANDLE} if(defined($self->{HANDLE}));
     return;
@@ -890,6 +986,22 @@ sub handle {
 Anansi::Component::addChannel('Anansi::DatabaseComponent', 'HANDLE' => 'handle');
 
 
+=head2 initialise
+
+Overrides L<Anansi::Class::initialise|Anansi::Class/"initialise">.  A virtual method.
+
+=cut
+
+
+sub initialise {
+    my ($self, %parameters) = @_;
+    Anansi::Actor->new(
+        PACKAGE => 'DBI',
+    );
+    $self->{STATEMENT} = {};
+}
+
+
 =head2 prepare
 
     my $PREPARATION = if(1 == Anansi::DatabaseComponent::prepare($OBJECT, undef,
@@ -1071,6 +1183,296 @@ sub rollback {
 Anansi::Component::addChannel('Anansi::DatabaseComponent', 'ROLLBACK' => 'rollback');
 
 
+=begin comment
+
+################################################################################
+
+=head2 script
+
+    my $result = $object->script(
+        undef,
+        SCRIPT => [
+            {
+                COMMAND => 'LOOP',
+                TEST => '',
+            }, [
+                {
+                },
+            ],
+        ],
+    );
+
+=over 4
+
+=item self I<(Blessed Hash, Required)>
+
+Either an object or a string of this namespace.
+
+=item channel I<(String, Required)>
+
+The abstract identifier of a subroutine.
+
+=item parameters I<(Hash, Required)>
+
+Named parameters.
+
+=over 4
+
+=item SCRIPT I<(Array, Required)>
+
+The SQL statements, control structures and external process triggers that are
+iterated through in sequence.
+
+=over 4
+
+=item I<(Array)>
+
+A sequence of statements like the I<SCRIPT> parameter.
+
+=item I<(Hash)>
+
+Either a SQL statement, control structure or external process trigger.
+
+=over 4
+
+=item COMMAND I<(String, Required)>
+
+A script command.
+
+=over 4
+
+=item B<"LOOP">
+
+Describes how to iterate over the subsequent sequence statement.
+
+=item B<"RUN">
+
+Describes how to perform an external subroutine.
+
+=item B<"SET">
+
+Describes how to define or redefine script variables.
+
+=item B<"SQL">
+
+Describes a specific SQL statement and it's interaction requirements.
+
+=item B<"TEST">
+
+Describes whether to perform the subsequent sequence statement.
+
+=back
+
+=item INPUT I<(Array B<or> String, Optional)>
+
+An array of strings or a string containing the variables or variable data.  Only
+used when I<COMMAND> contains B<"LOOP">, B<"RUN"> or B<"SQL">.
+
+=item NAME I<(String, Optional)
+
+A sequence name associated with the SQL statement, control structure or external
+process trigger.  When I<COMMAND> is B<"LOOP">, represents the name of a script
+variable hash that will contain the current I<ITERATION> of the loop starting at
+B<0> I<(zero)>, how many I<ITERATIONS> the loop is expected to perform a B<1>
+I<(one)> or greater or a B<0> I<(zero)> for indeterminate and the I<INPUT> value
+to use in the iteration or B<undef> if indeterminate.  When I<COMMAND> is B<"RUN">
+, represents the name of a script variable that will contain the results of the
+trigger.  When I<COMMAND> is B<"TEST">, represents the name of a script variable
+that will either contain the result of the current test or the name of a script
+variable that should be used to determine whether the sequence statement should
+be performed.
+
+=item SQL I<(String, Optional)>
+
+A SQL statement like that described in the I<statement> method's I<SQL>
+parameter.  Only used when I<COMMAND> contains B<"SQL">.
+
+=item STATEMENT I<(String, Optional)>
+
+The name associated with a SQL statement like that described in the I<statement>
+method's I<STATEMENT> parameter.  Only used when I<COMMAND> contains B<"SQL">.
+
+=item TRIGGER I<(String, Optional)>
+
+N/A
+
+=item VALUE I<Array B<or> Hash, Optional>
+
+Either an array of hashes or a hash representing specific script variables and
+variable data.  Only used when I<COMMAND> contains B<"LOOP">, B<"SET"> or B<"TEST">.
+When I<COMMAND> is B<"LOOP">, all of the referenced script variables must equate
+to the referenced values for the body of the loop to iterate.  When I<COMMAND>
+is B<"SET">, all of the referenced script variables are either declared or
+redeclared to contain the referenced values.  When I<COMMAND> is B<"TEST">, all of
+the referenced script variables must equate to the referenced values for the
+body of the if to iterate.
+
+=back
+
+=back
+
+=back
+
+=back
+
+Combines SQL statements together with a way to process them dynamically, making
+it possible to perform compound database tasks without needing to resort to
+using database functions or Perl subroutines.
+
+#=cut
+
+
+sub script {
+    my ($self, $channel, %parameters) = @_;
+    my ($VARIABLES);
+
+    sub scriptCommand {
+    }
+
+    sub scriptLocalBegin {
+        $VARIABLES = [] if(!defined($VARIABLES));
+        $VARIABLES = [] if(ref($VARIABLES) !~ /^ARRAY$/i);
+        for(my $index = 0; $index < scalar(@{$VARIABLES}); $index++) {
+            $VARIABLES[$index] = {} if(ref($VARIABLES[$index]) !~ /^HASH$/i);
+        }
+        push(@{$VARIABLES}, {});
+    }
+
+    sub scriptLocalEnd {
+        $VARIABLES = [] if(!defined($VARIABLES));
+        $VARIABLES = [] if(ref($VARIABLES) !~ /^ARRAY$/i);
+        pop(@{$VARIABLES}) if(0 < scalar(@{$VARIABLES}));
+        for(my $index = 0; $index < scalar(@{$VARIABLES}); $index++) {
+            $VARIABLES[$index] = {} if(ref($VARIABLES[$index]) !~ /^HASH$/i);
+        }
+    }
+
+    sub scriptGlobal {
+        return scriptLocal(0, (@_));
+    }
+
+    sub scriptLocal {
+        $VARIABLES = [] if(!defined($VARIABLES));
+        $VARIABLES = [] if(ref($VARIABLES) !~ /^ARRAY$/i);
+        for(my $index = 0; $index < scalar(@{$VARIABLES}); $index++) {
+            $VARIABLES[$index] = {} if(ref($VARIABLES[$index]) !~ /^HASH$/i);
+        }
+        my $DEPTH = scalar(@{$VARIABLES}) - 1;
+        if(0 < scalar(@_)) {
+            my $depth = shift(@_);
+            if(!defined($depth)) {
+            } elsif(ref($depth) !~ /^$/) {
+                return ;
+            } elsif($depth !~ /^[\-+]?\d+$/) {
+                return ;
+            } elsif($depth < 0) {
+                $DEPTH = scalar(@{$VARIABLES}) + $depth;
+                $DEPTH = 0 if($DEPTH < 0);
+            } elsif(scalar(@{$VARIABLES}) <= $depth) {
+            } else {
+                $DEPTH = $depth;
+            }
+        }
+        my %ALL_VARIABLES;
+        for(my $index = 0; $index <= $DEPTH; $index++) {
+            %ALL_VARIABLES = (%ALL_VARIABLES, %{$VARIABLES[$index]});
+        }
+        if(0 == scalar(@_)) {
+            return [( keys(%ALL_VARIABLES) )];
+        } elsif(1 == scalar(@_)) {
+            my ($key) = @_;
+            return if(!defined($key));
+            return if(ref($key) !~ /^$/);
+            return if(!defined($ALL_VARIABLES{$key}));
+            return $ALL_VARIABLES{$key};
+        } elsif(2 == scalar(@_)) {
+            my ($key, $value) = @_;
+            return 0 if(!defined($key));
+            return 0 if(ref($key) !~ /^$/);
+            if(defined($value)) {
+                ${$VARIABLES[$DEPTH]}{$key} = $value;
+                return 1;
+            }
+            if(defined(${$VARIABLES[$DEPTH]}{$key})) {
+                delete ${$VARIABLES[$DEPTH]}{$key};
+                return 1;
+            }
+            return 0;
+        }
+        return ;
+    }
+
+    sub scriptSequence {
+        my (@commands) = @_;
+        scriptLocalBegin();
+        my $sequence = {
+            COMMAND => 0,
+            COMMANDS => [( @commands )],
+        };
+        do {
+            last if(!scriptLocal('_', $sequence));
+            last if(!($sequence{COMMAND} < scalar(@{$sequence{COMMANDS}})));
+            scriptCommand();
+            $sequence{COMMAND}++;
+        } while($sequence{COMMAND} < scalar(@{$sequence{COMMANDS}}));
+        scriptLocalEnd();
+    }
+
+    sub scriptValidate {
+    }
+
+    sub scriptVariable {
+    }
+
+    sub block {
+        my (undef, @commands) = @_;
+        ${$_[0]}{'_'} = [] if(!defined(${$_[0]}{'_'}));
+        push(@{${$_[0]}{'_'}}, {});
+        ${${${$_[0]}{'_'}}[-1]}{COMMANDS} = [( @commands )];
+        for(my $index = 0; $index < scalar(@commands); $index++) {
+            my $command = $commands[$index];
+            if(ref($command) =~ /^ARRAY$/i) {
+                ${${${$_[0]}{'_'}}[-1]}{INDEX} = $INDEX;
+                block($_[0], (@{$command}));
+            }
+            next if(ref($command) !~ /^HASH$/i);
+            next if(!defined($command{COMMAND}));
+            next if(ref($command{COMMAND}) !~ /^$/);
+            if(!defined($command{NAME})) {
+            } elsif(ref($command{NAME}) !~ /^$/) {
+                delete $command{NAME};
+            } elsif($command{NAME} =~ /^\s*$/) {
+                delete $command{NAME};
+            } elsif(0 == 1) {
+            } else {
+                ${${${$_[0]}{'_'}}[-1]}{NAME} = $command{NAME};
+            }
+            if($command{COMMAND} =~ /^LOOP$/i) {
+            } elsif($command{COMMAND} =~ /^RUN$/i) {
+            } elsif($command{COMMAND} =~ /^SET$/i) {
+            } elsif($command{COMMAND} =~ /^SQL$/i) {
+            } elsif($command{COMMAND} =~ /^TEST$/i) {
+            }
+        }
+        pop(@{${$_[0]}{'_'}});
+        delete ${$_[0]}{'_'} if(0 == scalar(@{${$_[0]}{'_'}}));
+    }
+    return if(!defined($parameters{SCRIPT}));
+    return if(ref($parameters{SCRIPT}) !~ /^ARRAY$/i);
+    my $variables = {};
+    foreach my $key (keys(%parameters)) {
+        $variables{$key} = $parameters{$key};
+    }
+    block($variables, (@{$parameters{SCRIPT}}));
+}
+
+Anansi::Component::addChannel('Anansi::DatabaseComponent', 'SCRIPT' => 'script');
+
+################################################################################
+
+=end comment
+
+
 =head2 statement
 
     my $result = Anansi::DatabaseComponent::statement($OBJECT, undef,
@@ -1266,6 +1668,10 @@ An array of strings or a single string containing either the namespace of a
 valid database driver or the name of a database driver that should be looked for
 among the installed modules.
 
+=item HANDLE I<DBI::db, Optional>
+
+An existing database connection handle.
+
 =back
 
 =back
@@ -1285,21 +1691,36 @@ sub validate {
     $package = ref($self) if(ref($self) !~ /^$/);
     my %modules = Anansi::Actor->modules();
     return 0 if(!defined($modules{'Bundle::DBI'}));
+    my $HANDLE_DRIVER;
+    if(defined($parameters{HANDLE})) {
+        return 0 if(ref($parameters{HANDLE}) !~ /^DBI::db$/);
+        my $driver = $parameters{HANDLE}->get_info(17);
+        return 0 if(!defined($driver));
+        $HANDLE_DRIVER = $driver;
+    }
     if(!defined($parameters{DRIVER})) {
         if(defined($parameters{DRIVERS})) {
             $parameters{DRIVERS} = [( $parameters{DRIVERS} )] if(ref($parameters{DRIVERS}) =~ /^$/);
             return 0 if(ref($parameters{DRIVERS}) !~ /^ARRAY$/i);
-            my %reduced = map { lc($_) => $modules{$_} } (keys(%modules));
-            foreach my $DRIVER (@{$parameters{DRIVERS}}) {
-                return 0 if(ref($DRIVER) !~ /^$/);
-                return 1 if(defined($modules{$DRIVER}));
-                return 1 if(defined($modules{'DBD::'.$DRIVER}));
-                return 1 if(defined($modules{'Bundle::DBD::'.$DRIVER}));
-                return 1 if(defined($reduced{lc($DRIVER)}));
-                return 1 if(defined($reduced{lc('DBD::'.$DRIVER)}));
-                return 1 if(defined($reduced{lc('Bundle::DBD::'.$DRIVER)}));
+            if(defined($HANDLE_DRIVER)) {
+                foreach my $DRIVER (@{$parameters{DRIVERS}}) {
+                    return 0 if(ref($DRIVER) !~ /^$/);
+                    return 1 if($DRIVER eq $HANDLE_DRIVER);
+                }
+            } else {
+                my %reduced = map { lc($_) => $modules{$_} } (keys(%modules));
+                foreach my $DRIVER (@{$parameters{DRIVERS}}) {
+                    return 0 if(ref($DRIVER) !~ /^$/);
+                    return 1 if(defined($modules{$DRIVER}));
+                    return 1 if(defined($modules{'DBD::'.$DRIVER}));
+                    return 1 if(defined($modules{'Bundle::DBD::'.$DRIVER}));
+                    return 1 if(defined($reduced{lc($DRIVER)}));
+                    return 1 if(defined($reduced{lc('DBD::'.$DRIVER)}));
+                    return 1 if(defined($reduced{lc('Bundle::DBD::'.$DRIVER)}));
+                }
             }
             return 0;
+        } elsif(defined($HANDLE_DRIVER)) {
         } elsif(!defined($modules{'Bundle::DBD'})) {
             return 0;
         }
@@ -1328,23 +1749,46 @@ sub validate {
             $found = 0;
         }
         return 0 if(!$found);
-        my %reduced = map { lc($_) => $modules{$_} } (keys(%modules));
-        foreach my $DRIVER (@{$parameters{DRIVERS}}) {
-            return 1 if(defined($modules{$DRIVER}));
-            return 1 if(defined($modules{'DBD::'.$DRIVER}));
-            return 1 if(defined($modules{'Bundle::DBD::'.$DRIVER}));
-            return 1 if(defined($reduced{lc($DRIVER)}));
-            return 1 if(defined($reduced{lc('DBD::'.$DRIVER)}));
-            return 1 if(defined($reduced{lc('Bundle::DBD::'.$DRIVER)}));
+        if(defined($HANDLE_DRIVER)) {
+            foreach my $DRIVER (@{$parameters{DRIVERS}}) {
+                return 0 if(ref($DRIVER) !~ /^$/);
+                return 1 if($DRIVER eq $HANDLE_DRIVER);
+            }
+        } else {
+            my %reduced = map { lc($_) => $modules{$_} } (keys(%modules));
+            foreach my $DRIVER (@{$parameters{DRIVERS}}) {
+                return 1 if(defined($modules{$DRIVER}));
+                return 1 if(defined($modules{'DBD::'.$DRIVER}));
+                return 1 if(defined($modules{'Bundle::DBD::'.$DRIVER}));
+                return 1 if(defined($reduced{lc($DRIVER)}));
+                return 1 if(defined($reduced{lc('DBD::'.$DRIVER)}));
+                return 1 if(defined($reduced{lc('Bundle::DBD::'.$DRIVER)}));
+            }
         }
         return 0;
     } elsif(defined($modules{$parameters{DRIVER}})) {
+        if(defined($HANDLE_DRIVER)) {
+            return 0 if($parameters{DRIVER} ne $HANDLE_DRIVER);
+        }
     } elsif(defined($modules{'DBD::'.$parameters{DRIVER}})) {
+        if(defined($HANDLE_DRIVER)) {
+            return 0 if('DBD::'.$parameters{DRIVER} ne $HANDLE_DRIVER);
+        }
     } elsif(!defined($modules{'Bundle::DBD::'.$parameters{DRIVER}})) {
         my %reduced = map { lc($_) => $modules{$_} } (keys(%modules));
         if(defined($reduced{lc($parameters{DRIVER})})) {
+            if(defined($HANDLE_DRIVER)) {
+                return 0 if(lc($parameters{DRIVER}) ne lc($HANDLE_DRIVER));
+            }
         } elsif(defined($reduced{lc('DBD::'.$parameters{DRIVER})})) {
-        } elsif(!defined($reduced{lc('Bundle::DBD::'.$parameters{DRIVER})})) {
+            if(defined($HANDLE_DRIVER)) {
+                return 0 if(lc('DBD::'.$parameters{DRIVER}) ne lc($HANDLE_DRIVER));
+            }
+        } elsif(defined($reduced{lc('Bundle::DBD::'.$parameters{DRIVER})})) {
+            if(defined($HANDLE_DRIVER)) {
+                return 0 if(lc('Bundle::DBD::'.$parameters{DRIVER}) ne lc($HANDLE_DRIVER));
+            }
+        } else {
             return 0;
         }
     }
